#!/usr/bin/env python
# encoding: utf-8
"""
steam.py

Created by Scott on 2013-12-28.
Copyright (c) 2013 Scott Rice. All rights reserved.

Represents the local steam installation.
"""

import sys
import os

def _is_mac():
    return sys.platform == 'darwin'

def _is_windows():
    return sys.platform != 'darwin' and 'win' in sys.platform
    
def _is_linux():
    return sys.platform.startswith('linux')

def _windows_steam_location():
    if not _is_windows():
        return
    import _winreg as registry
    key = registry.CreateKey(registry.HKEY_CURRENT_USER,"Software\Valve\Steam")
    return registry.QueryValueEx(key,"SteamPath")[0]

class Steam(object):
    
    def __init__(self, steam_location=None):
        # If no steam_location was provided but we are on Windows, then we can
        # find Steam's location by looking in the registry
        if not steam_location and _is_windows():
            steam_location = _windows_steam_location()
        self.steam_location = steam_location

    def userdata_location(self):
        if _is_windows():
            return os.path.join(self.steam_location, "userdata")
        elif _is_mac():
            return os.path.join(os.path.expanduser("~"),
                                "Library",
                                "Application Support",
                                "Steam",
                                "userdata"
            )
        elif _is_linux():
            return os.path.join(os.path.expanduser("~"),
                                ".local",
                                "share",
                                "Steam",
                                "userdata"
            )
        else:
            raise EnvironmentError("Running on unsupported environment %s" % sys.platform)



#!/usr/bin/env python
# encoding: utf-8
"""
user.py

Created by Scott on 2013-12-28.
Copyright (c) 2013 Scott Rice. All rights reserved.
"""

import sys
import os

import steam
from _shortcut_parser import ShortcutParser
from _shortcut_generator import ShortcutGenerator

# Information about SteamIDs and conversion between them found here:
# https://developer.valvesoftware.com/wiki/SteamID
#
# In short:
# CommunityID32 = Z*2 + Y
# CommunityID64 = Z*2 + V + Y
# Therefore: CommunityID64 = CommunityID32 + V
# Where V is the Steam64 Identifier for the account type (0x0110000100000000
# for individuals, 0x0170000000000000 for groups)
individual_account_type_identifier = 0x0110000100000000

def _community_id_is_64(communityid):
    return communityid > individual_account_type_identifier

def _community_id_32_from_64(communityid64):
    return communityid64 - individual_account_type_identifier

def _community_id_64_from_32(communityid32):
    return communityid32 + individual_account_type_identifier

class User(object):

    @staticmethod
    def local_users(steam):
        """Returns an array of user ids for users on the filesystem"""
        # Any users on the machine will have an entry inside of the userdata
        # folder. As such, the easiest way to find a list of all users on the
        # machine is to just list the folders inside userdata
        ids = []
        userdata_dir = steam.userdata_location()
        for entry in os.listdir(userdata_dir):
            if os.path.isdir(os.path.join(userdata_dir,entry)):
                ids.append(int(entry))
        return ids
    
    def __init__(self, steam, userid):
        self.steam = steam

        if _community_id_is_64(userid):
            self.id32 = _community_id_32_from_64(userid)
            self.id64 = userid
        else:
            self.id32 = userid
            self.id64 = _community_id_64_from_32(userid)

        self.shortcuts = self._load_shortcuts()

    def _user_config_directory(self):
        return os.path.join(
            self.steam.userdata_location(),
            str(self.id32),
            "config"
        )
    
    def _load_shortcuts(self):
        try:
            parsed_shortcuts = ShortcutParser().parse(self.shortcuts_file())
        except IOError:
            parsed_shortcuts = []
        if parsed_shortcuts == None:
            # TODO: Raise a decent error
            print "Parsing error on file: %s" % file
            parsed_shortcuts = []
        return parsed_shortcuts

    def shortcuts_file(self):
        """Returns a path to this users shortcuts.vdf file"""
        return os.path.join(self._user_config_directory(), "shortcuts.vdf")
    
    def grid_directory(self):
        """Returns a path to this users grid image directory, where custom
        grid images are stored"""
        return os.path.join(self._user_config_directory(), "grid")
    
    def save_shortcuts(self):
        # Write shortcuts to file
        contents = ShortcutGenerator().to_string(self.shortcuts)
        f = open(self.shortcuts_file(), "w")
        f.write(contents)
        f.close()



#!/usr/bin/env python
# encoding: utf-8
"""
shortcut.py

Created by Scott on 2013-12-28.
Copyright (c) 2013 Scott Rice. All rights reserved.
"""

import sys
import os

import game
from _crc_algorithms import Crc

class Shortcut(game.Game):

    def __init__(self, name, exe, startdir, icon="", tag=""):
        self.name = name
        self.exe = exe
        self.startdir = startdir
        self.icon = icon
        self.tag = tag

    def appid(self):
        algorithm = Crc(width = 32, poly = 0x04C11DB7, reflect_in = True, xor_in = 0xffffffff, reflect_out = True, xor_out = 0xffffffff)
        crc_input = ''.join([self.exe,self.name])
        top_32 = algorithm.bit_by_bit(crc_input) | 0x80000000
        full_64 = (top_32 << 32) | 0x02000000
        return str(full_64)


#!/usr/bin/env python
# encoding: utf-8
"""
_shortcut_generator.py

Created by Scott on 2013-12-29.
Copyright (c) 2013 Scott Rice. All rights reserved.
"""

import sys
import os

x00 = u'\x00'
x01 = u'\x01'
x08 = u'\x08'
x0a = u'\x0a'

class ShortcutGenerator(object):

    def to_string(self,shortcuts):
        string = x00 + 'shortcuts' + x00 + self.generate_array_string(shortcuts) + x08 + x08 + x0a
        # rstrip is to remove the eol character that is automatically added.
        # According to vim the files I got from steam don't have the eol character
        return unicode(string).rstrip()
    
    def generate_array_string(self,shortcuts):
        string = ""
        for i in range(len(shortcuts)):
            shortcut = shortcuts[i]
            string += x00 + str(i) + x00 + self.generate_shortcut_string(shortcut)
        return string

    def generate_shortcut_string(self,shortcut):
        string = ""
        string += self.generate_keyvalue_pair("AppName",shortcut.name)
        string += self.generate_keyvalue_pair("Exe",shortcut.exe)
        string += self.generate_keyvalue_pair("StartDir",shortcut.startdir)
        string += self.generate_keyvalue_pair("icon",shortcut.icon)
        # Tags seem to be a special case. It seems to be a key-value pair just
        # like all the others, except it doesnt start with a x01 character. It
        # also seems to be an array, even though Steam wont let more than one
        # be used. I am just going to use a special function to represent this
        # strange case
        string += self.generate_tags_string(shortcut.tag)
        string += x08
        return string

    # The 'more' variable was for when I used this function to generate tags
    # I'm not sure if tags are a special case, or if dictionaries keyvalues are
    # supposed to end in x00 when there are more and x08 when there arent. Since
    # I am not sure, I am going to leave the code in for now
    def generate_keyvalue_pair(self,key,value,more=True):
        return x01 + key + x00 + value + (x00 if more else x08)

    def generate_tags_string(self,tag):
        string = x00 + "tags" + x00
        if tag == "":
            string += x08
        else:
            string += self.generate_tag_array_string([tag])
        return string

    def generate_tag_array_string(self,tags):
        string = ""
        for i in range(len(tags)):
            tag = tags[i]
            string += x01 + str(i) + x00 + str(tag) + x00 + x08
        return string


#!/usr/bin/env python
# encoding: utf-8
"""
__init__.py

Created by Scott on 2013-12-28.
Copyright (c) 2013 Scott Rice. All rights reserved.
"""

__all__ = [
    "game",
    "shortcut",
    "steam",
    "user",
]



#!/usr/bin/env python
# encoding: utf-8
"""
_shortcut_parser.py

Created by Scott on 2013-12-29.
Copyright (c) 2013 Scott Rice. All rights reserved.
"""

import sys
import os
import re

import shortcut

class ShortcutParser(object):
    
    def parse(self, path):
        if not os.path.exists(path):
            raise IOError("Shortcuts file '%s' does not exist" % path)
        file_contents = open(path, "r").read()
        return self.match_base(file_contents)

    def match_base(self,string):
            match = re.match(ur"\u0000shortcuts\u0000(.*)\u0008\u0008$",string, re.IGNORECASE)
            if match:
                return self.match_array_string(match.groups()[0])
            else:
                return None

    def match_array_string(self,string):
        # Match backwards (aka match last item first)
        if string == "":
            return []
        # One side effect of matching this way is we are throwing away the
        # array index. I dont think that it is that important though, so I am
        # ignoring it for now
        shortcuts = []
        while True:
            match = re.match(ur"(.*)\u0000[0-9]+\u0000(\u0001AppName.*)\u0008",string, re.IGNORECASE)
            if match:
                groups = match.groups()
                string = groups[0]
                shortcuts.append(self.match_shortcut_string(groups[1]))
            else:
                shortcuts.reverse()
                return shortcuts

    def match_shortcut_string(self,string):
        # I am going to cheat a little here. I am going to match specifically
        # for the shortcut string (Appname, Exe, StartDir, etc), as oppposed
        # to matching for general Key-Value pairs. This could possibly create a
        # lot of work for me later, but for now it will get the job done
        match = re.match(ur"\u0001AppName\u0000(.*)\u0000\u0001Exe\u0000(.*)\u0000\u0001StartDir\u0000(.*)\u0000\u0001icon\u0000(.*)\u0000\u0000tags\u0000(.*)\u0008",string, re.IGNORECASE)
        if match:
            # The 'groups' that are returned by the match should be the data
            # contained in the file. Now just make a Shortcut out of that data
            groups = match.groups()
            appname = groups[0]
            exe = groups[1]
            startdir = groups[2]
            icon = groups[3]
            tags = self.match_tags_string(groups[4])
            return shortcut.Shortcut(appname,exe,startdir,icon,tags)
        else:
            return None

    def match_tags_string(self,string):
        match = re.match(ur"\u00010\u0000(.*)\u0000",string)
        if match:
            groups = match.groups()
            return groups[0]
        else:
            return ""


from setuptools import setup

setup(
    name='pysteam',
    version='0.1.1',
    description='Python library to work with Steam',
    url='http://github.com/scottrice/pysteam',
    author='Scott Rice',
    author_email='',
    license='MIT',
    packages=['pysteam'],
    install_requires=[
    ],
    data_files=[
    ],
    dependency_links=[
    ],
    zip_safe=False,
    test_suite='nose.collector',
    tests_require=[
        'nose',
        'mock',
    ],
)



#!/usr/bin/env python
# encoding: utf-8
"""
game.py

Created by Scott on 2013-12-28.
Copyright (c) 2013 Scott Rice. All rights reserved.
"""

import sys
import os
import shutil

class Game(object):

    @staticmethod
    def valid_custom_image_extensions():
        return ['.png', '.jpg', '.jpeg', '.tga']

    def __init__(self, user, appid):
        self.user = user
        self._appid = appid

    def _custom_image_path(self, extension):
        filename = '%s%s' % (self.appid(), extension)
        return os.path.join(self.user.grid_directory(), filename)

    def appid(self):
        return str(self._appid)

    def custom_image(self):
        """Returns the path to the custom image set for this game, or None if
        no image is set"""
        for ext in self.valid_custom_image_extensions():
            image_location = self._custom_image_path(ext)
            if os.path.isfile(image_location):
                return image_location
        return None

    def set_image(self, image_path):
        """Sets a custom image for the game. `image_path` should refer to
        an image file on disk"""
        _, ext = os.path.splitext(image_path)
        shutil.copy(image_path, self._custom_image_path(ext))
